class FeatureParser:
    """
    Single Unit which is a part of geojson features array
    """

    def __init__(self):
        pass


class FeaturesParser:
    """
    GeoJSON features parser representation
    """

    def __init__(self):
        pass
