MESSAGES = dict({
    "1": "\n\nData provided for method __init__() of class Feature \n"
         "was not correct to create dict() object",
    "2": "\n\nFeature object is not a valid geojson feature object, \n"
         "one of the following failed, geometry, properties or type field are missing \n",
    "3": "\n\nCoordinates Array Items must be of float type\n"
})